
;
;       Matrix3DSH4.asm - 3D Vector Matrix Function
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to work with the exercises for this book.
;
;  Created - 29 Apr 2002 - James Leiterman
;
;  Last Modification
;
;  stab=4
;



;
;void vmp_MatrixMulGeneric( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx )
;                                       r4                     r5                     r6
;               |D| = |A| x |B|
;
;       Permission for inclusion into the book granted by Hitachi.
;       Modified from _matrx4mul
;
;       [0 1 2 3]
;       [4 5 6 7]
;       [8 9 A B]
;       [C D E F]
;

        .EXPORT _vmp_MatrixMulSH4
_vmp_MatrixMulSH4:

ASSERT_PTR4(r4)                         ; pvD
ASSERT_PTR4(r5)                         ; pvA
ASSERT_PTR4(r6)                         ; pvB

        add     #-12,r15                        ; Adjust Stack

        mov.l   r6,@(8,r15)             ; Save ptr to |B| on local stack
        mov.l   r5,@(4,r15)             ;    "        |A|       "
        mov.l   r4,@r15                 ;    "        |D|       "

        mov.l   @(8,r15),r2             ; =|B|

        frchg                                   ; Set Matrix elements in background bank
        fmov.s  @r2+,fr0                ; [b0      ]
        fmov.s  @r2+,fr1                ; [  b1    ]
        fmov.s  @r2+,fr2                ; [    b2  ]
        fmov.s  @r2+,fr3                ; [      b3]

        fmov.s  @r2+,fr4                ; [b4      ]
        fmov.s  @r2+,fr5                ; [  b5    ]
        fmov.s  @r2+,fr6                ; [    b6  ]
        fmov.s  @r2+,fr7                ; [      b7]
        
        fmov.s  @r2+,fr8                ; [b8      ]
        fmov.s  @r2+,fr9                ; [  b9    ]
        fmov.s  @r2+,fr10               ; [    bA  ]
        fmov.s  @r2+,fr11               ; [      bB]
        
        fmov.s  @r2+,fr12               ; [bC      ]
        fmov.s  @r2+,fr13               ; [  bD    ]
        fmov.s  @r2+,fr14               ; [    bE  ]
        fmov.s  @r2+,fr15               ; [      bF]
        frchg

        mov.l   @(4,r15),r3             ; =|A|
        mov.l   @r15,r1                 ; =|D|

                ;       1st row           [0 1 2 3]

        fmov.s  @r3+,fr0                ; [a0      ]
        fmov.s  @r3+,fr1                ; [  a1    ]
        fmov.s  @r3+,fr2                ; [    a2  ]
        fmov.s  @r3+,fr3                ; [      a3]

        ftrv    xmtrx,fv0
        add     #16,r1                          ; skip past end of 1st row of |D|

        fmov.s  fr3,@-r1                ; [      d3] (Save 1st row)
        fmov.s  fr2,@-r1                ; [    d2  ]
        fmov.s  fr1,@-r1                ; [  d1    ]
        fmov.s  fr0,@-r1                ; [d0      ]

                ;       2nd row                   [4 5 6 7]

        fmov.s  @r3+,fr0                ; [a4      ]
        fmov.s  @r3+,fr1                ; [  a5    ]
        fmov.s  @r3+,fr2                ; [    a6  ]
        fmov.s  @r3+,fr3                ; [      a7]

        ftrv    xmtrx,fv0
        add     #32,r1                          ; skip past end of 2nd row of |D|

        fmov.s  fr3,@-r1                ; [      d7] (Save 2nd row)
        fmov.s  fr2,@-r1                ; [    d6  ]
        fmov.s  fr1,@-r1                ; [  d5    ]
        fmov.s  fr0,@-r1                ; [d4      ]

                ;       3rd row                   [8 9 10(A) 11(B)]

        fmov.s  @r3+,fr0                ; [a8      ]
        fmov.s  @r3+,fr1                ; [  a9    ]
        fmov.s  @r3+,fr2                ; [    aA  ]
        fmov.s  @r3+,fr3                ; [      aB]

        ftrv    xmtrx,fv0
        add     #32,r1                          ; skip past end of 3rd row of |D|

        fmov.s  fr3,@-r1                ; [      dB] (Save 3rd row)
        fmov.s  fr2,@-r1                ; [    dA  ]
        fmov.s  fr1,@-r1                ; [  d9    ]
        fmov.s  fr0,@-r1                ; [d8      ]

                ;       4th row                   [12(C) 13(D) 14(E) 15(F)]

        fmov.s  @r3+,fr0                ; [aC      ]
        fmov.s  @r3+,fr1                ; [  aD    ]
        fmov.s  @r3+,fr2                ; [    aE  ]
        fmov.s  @r3+,fr3                ; [      aF]

        ftrv    xmtrx,fv0
        add     #32,r1                          ; skip to end of 4th row of |D|

        fmov.s  fr3,@-r1                ; [      dF] (Save 4th row)
        fmov.s  fr2,@-r1                ; [    dE  ]
        fmov.s  fr1,@-r1                ; [  dD    ]
        fmov.s  fr0,@-r1                ; [dC      ]

        add     #12,r15                         ; Unadjust stack
        rts
        nop                                             ; nop = Return Delay Slot (Executed before return)
